package nce.editor;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.IStorageEditorInput;
import org.eclipse.ui.editors.text.TextFileDocumentProvider;
import org.gjt.sp.util.Log;

public class ColoringEditorDocumentProvider extends TextFileDocumentProvider {

	private ColoringEditorPartitionScanner fScanner;
	
	@Override
	protected FileInfo createFileInfo(Object element) throws CoreException {
		
		FileInfo info = super.createFileInfo(element);
    	if(info==null){
    		info = createEmptyFileInfo();
    	}
    	IDocument document = info.fTextFileBuffer.getDocument();
    	if (document != null) {

    		initializeDocument(document, getFilename(element));
    	}
    	return info;
	}

	private void initializeDocument(IDocument document, String filename) {
		
		IDocumentPartitioner partitioner = createPartitioner(filename);
		partitioner.connect(document);
		document.setDocumentPartitioner(partitioner);
		//printPartitions(document, partitioner.computePartitioning(0, document.getLength()));
		
	}

	private IDocumentPartitioner createPartitioner(String filename) {

		fScanner = new ColoringEditorPartitionScanner(filename);
		ColoringEditorPartitioner partitioner = new ColoringEditorPartitioner(fScanner, TokenType.getTokenTypeNames());
		
		return partitioner;
	}

	private String getFilename(Object element) {

		if (element instanceof IPathEditorInput) {
			return ((IPathEditorInput) element).getName();

		} else if (element instanceof IStorageEditorInput) {

			try {
				return ((IStorageEditorInput) element).getStorage().getName();
			} catch (CoreException e) {
				e.printStackTrace();
			}

		} else if (element instanceof IEditorInput) {
			return ((IEditorInput) element).getName();

		}
		return "";
	}

	public String getModeDisplayName() {

		return fScanner != null ? fScanner.getModeName() : null;
	}

	@SuppressWarnings("unused")
	private void printPartitions(IDocument document, ITypedRegion[] partitions)
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append("\n---------------------------\n\n\n");

        for (int i = 0; i < partitions.length; i++)
        {
            try
            {
                buffer.append("Partition type: " 
                  + partitions[i].getType() 
                  + ", offset: " + partitions[i].getOffset()
                  + ", length: " + partitions[i].getLength());
                buffer.append("\n");
                buffer.append("Text:\n");
                buffer.append(document.get(partitions[i].getOffset(), 
                 partitions[i].getLength()));
                buffer.append("\n---------------------------\n\n\n");
            }
            catch (BadLocationException e)
            {
                e.printStackTrace();
            }
        }
        System.out.print(buffer);
        Log.log(Log.DEBUG, this, buffer.toString());
    }    
	
}